#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <fmi2Functions.h>
#include <fmi2FunctionTypes.h>
#include <Variables.hpp>

// global map array that matches each variable of the bip models with their name
map<string,int> int_variables;
map<string,double> real_variables;
map<string,int> boolean_variables;
map<string,string> string_variables;
map<int,string> inverted_map;

int main(int argc, char **argv) {

    int ret = EXIT_SUCCESS;
    int NUM_ITERATION = 50;
    void *handle;
  
    fmi2InstantiateTYPE* (fmi2Instantiate);
    fmi2EnterInitializationModeTYPE* (fmi2EnterInitializationMode);
    fmi2ExitInitializationModeTYPE* (fmi2ExitInitializationMode);
    fmi2SetIntegerTYPE* (fmi2SetInteger);
    fmi2GetIntegerTYPE* (fmi2GetInteger);
    fmi2DoStepTYPE* (fmi2DoStep);
   
    char *error;
    handle = dlopen ("./libProject.so", RTLD_LAZY);
    if (!handle) {
        fprintf (stderr, "%s\n", dlerror());
        exit(1);
    }
    dlerror();    /* Clear any existing error */
    
    fmi2Instantiate = (fmi2InstantiateTYPE*) dlsym(handle, "fmi2Instantiate");
    fmi2EnterInitializationMode = (fmi2EnterInitializationModeTYPE*) dlsym(handle, "fmi2EnterInitializationMode");
    fmi2ExitInitializationMode = (fmi2ExitInitializationModeTYPE*) dlsym(handle, "fmi2ExitInitializationMode");
    fmi2SetInteger = (fmi2SetIntegerTYPE*) dlsym(handle, "fmi2SetInteger");
    fmi2GetInteger = (fmi2GetIntegerTYPE*) dlsym(handle, "fmi2GetInteger");
    fmi2DoStep = (fmi2DoStepTYPE*) dlsym(handle, "fmi2DoStep");
    
    if ((error = dlerror()) != NULL)  {
        fprintf (stderr, "%s\n", error);
        exit(1);
    }
    
    cout<<"CALL fmi2Instantiate...\n";
    
    fmi2Component c = (*fmi2Instantiate)("prModel", fmi2CoSimulation , "prModel", "resource/location", NULL, true, true);
    
    inverted_map[100] = "ROOT.environment._id__inputIndex";
    inverted_map[101] = "ROOT.environment._id__time";
    inverted_map[102] = "ROOT.camera._id__inputIndex";
    inverted_map[103] = "ROOT.perception._id__inputIndex";
    inverted_map[104] = "ROOT.perception._id__result";
    inverted_map[105] = "ROOT.controller._id__speed";
    inverted_map[106] = "ROOT.controller._id__speedLimit";
    inverted_map[107] = "ROOT.controller._id__deltaSpeed";
    inverted_map[108] = "ROOT.controller._id__zero";
    inverted_map[109] = "ROOT.brake._id__deltaSpeed";
    inverted_map[110] = "ROOT.throttle._id__deltaSpeed";
    inverted_map[111] = "ROOT.speedSensor._id__speed";
    inverted_map[112] = "ROOT.speedSensor._id__deltaSpeed";
    
    const unsigned int vr[] = {100,101,102,103,104,105,106,107,108,109,110,111,112};
    const int values[] = {1,10,0,0,0,0,0,0,0,0,0,100,0};
    
    cout<<"CALL fmi2SetInteger...\n";
    (*fmi2SetInteger)(c, vr, 13, values);
    
    fmi2Integer values2[] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
    
    cout<<"CALL fmi2EnterInitializationMode...\n";
    (*fmi2EnterInitializationMode)(c);
    
    cout<<"CALL fmi2ExitInitializationMode...\n";
    //(*fmi2ExitInitializationMode)(c);
    
    int i=0;
    while(i<NUM_ITERATION){
        i++;
        cout<<i<<"\n";
        cout<<"CALL fmi2GetInteger...\n";
        fmi2GetInteger(c, vr, 13, values2);
        cout<<"CALL fmi2DoStep...\n";
        fmi2DoStep(c, 0.5,0.6, true);
     }
    
   dlclose(handle);
    return ret;
}
